@@ -1055,6 +1055,12 @@ else:
 
 
 csz = settings.get_settings("DB_CACHE_ENTRIES")
+cmp = settings.get_settings("DB_MAX_MEMORY")
+try:
+	import psutil
+	use_psutil = True
+except:
+	use_psutil = False
 
 cache_query = lru.LRU(csz)
 cache_query_perm = lru.LRU(csz)
@@ -1087,7 +1093,7 @@ def log_stats():
 	logstr = "{name}: {hitsperm} Perm Hits, {hitstmp} Tmp Hits, {misses} Misses; Current Size: {sizeperm}/{sizetmp}"
 	for s in (cachestats["cache_query"],cachestats["cache_aggregate"]):
 		log(logstr.format(name=s["name"],hitsperm=s["hits_perm"],hitstmp=s["hits_tmp"],misses=s["misses"],
-		sizeperm=len(s["objperm"]),sizetmp=len(s["objtmp"])))
+		sizeperm=len(s["objperm"]),sizetmp=len(s["objtmp"])),module="debug")
 
 def db_query_cached(**kwargs):
 	global cache_query, cache_query_perm
@@ -1118,6 +1124,10 @@ def db_query_cached(**kwargs):
 		result = db_query_full(**kwargs)
 		if eligible_permanent_caching: cache_query_perm[key] = result
 		elif eligible_temporary_caching: cache_query[key] = result
+
+		if use_psutil:
+			reduce_caches_if_low_ram()
+
 		return result
 
 
@@ -1151,6 +1161,9 @@ def db_aggregate_cached(**kwargs):
 		if eligible_permanent_caching: cache_aggregate_perm[key] = result
 		elif eligible_temporary_caching: cache_aggregate[key] = result
 
+		if use_psutil:
+			reduce_caches_if_low_ram()
+
 		return result
 
 def invalidate_caches():
@@ -1159,6 +1172,21 @@ def invalidate_caches():
 	cache_aggregate.clear()
 	log("Database caches invalidated.")
 
+def reduce_caches(to=0.75):
+	global cache_query, cache_aggregate, cache_query_perm, cache_aggregate_perm
+	for c in cache_query, cache_aggregate, cache_query_perm, cache_aggregate_perm:
+		currentsize = len(c)
+		targetsize = int(currentsize * to)
+		c.set_size(targetsize)
+		c.set_size(csz)
+
+def reduce_caches_if_low_ram():
+	ramprct = psutil.virtual_memory().percent
+	if ramprct > cmp:
+		log("{prct}% RAM usage, reducing caches!".format(prct=ramprct),module="debug")
+		ratio = (cmp / ramprct) ** 3
+		reduce_caches(to=ratio)
+
 ####
 ## Database queries
 ####
